/**
 * KivaCRM Rest Api - JS Client V1.0.0
 */
(function (name, definition) {
    var theModule = definition(),
        hasDefine = typeof define === 'function',
        hasExports = typeof module !== 'undefined' && module.exports;

    if (hasDefine) { // AMD Module
        define(theModule);

    } else if (hasExports) { // Node.js Module
        module.exports = theModule;

    } else { // Global
        var scope = this || window;
        scope[name] = theModule;
    }
})('kvapi', function () {
    /**
     *
     * @param url
     * @constructor
     */
    function KivaApi(url) {
        this._url = url;
        this._token = null;
    }

    /**
     * set api url
     * @param url
     */
    KivaApi.prototype.setUrl = function (url) {
        this._url = url;
    };

    /**
     *
     * @param token
     */
    KivaApi.prototype.setAccessToken = function (token) {

        this._token = token;


    };

    /**
     * get ajax object
     * @returns {*}
     */
    KivaApi.prototype.getXhr = function () {
        var options = [function () {
                return new XMLHttpRequest();
            }, function () {
                return new ActiveXObject('MSXML2.XMLHTTP.3.0');
            }, function () {
                return new ActiveXObject('MSXML2.XMLHTTP');
            }, function () {
                return new ActiveXObject('Microsoft.XMLHTTP');
            }], i = 0,
            len = options.length,
            xhr;

        for (; i < len; ++i) {
            try {
                xhr = options[i]();
                break;
            } catch (e) {
            }
        }


        return xhr;
    };

    /**
     * rest api request
     * @param method
     * @param endpoint
     * @param params
     * @returns {*}
     */
    KivaApi.prototype.request = function (method, endpoint, params) {
        var me = this;
        var xhr = this.getXhr(),
            useXhr2 = xhr.upload !== null,
            url = this._url + endpoint,
            data;

        params = params || {};
        method = method.toUpperCase();

        var qs = [];

        if (method == "GET") {
            for (var key in params) {
                if (params.hasOwnProperty(key)) {
                    if (params[key] === false) params[key] = 0;
                    qs.push(encodeURI(key) + "=" + encodeURI(params[key]));
                }
            }
        }

        // authorization header causes preflight request

        qs.push("access_token=" + encodeURIComponent(this._token));
        qs = qs.join("&");

        url += "?" + qs;

        if (params instanceof FormData) {
            data = params;
        } else {
            data = JSON.stringify(params);
        }

        // open the request
        xhr.open(method, url);

        // require OPTIONS request
        //xhr.setRequestHeader("Authorization", "Bearer " + token);

        xhr.withCredentials = true;

        var promise = new Promise(function (resolve, reject) {
                xhr[useXhr2 ? 'onload' : 'onreadystatechange'] = function (request) {
                    if (xhr.readyState == 4) {
                        var response = xhr.responseText != "" ? JSON.parse(xhr.responseText) : {success: true};

                        var headerInfo = xhr.getAllResponseHeaders().split('\n').filter(Boolean),
                            limit = headerInfo.length, headers = {}, i, k;
                        for (i = 0; i < limit; i++) {
                            k = headerInfo[i].split(":");
                            headers[k[0].trim()] = (k[1] + "").trim();
                        }

                        if (response.success) {
                            me.tokenExpired = false;

                            resolve({
                                response: response,
                                headers: headers,
                                status: {
                                    code: xhr.status,
                                    message: xhr.statusText
                                }
                            });
                        } else {
                            if (response.error == "INVALID_TOKEN" && me.onTokenExpire && !me.tokenExpired) {
                                me.tokenExpired = true;
                                me.onTokenExpire(function () {
                                    me.request(method, endpoint, params)
                                        .then(function (rs) {
                                            if (rs.success) {
                                                me.tokenExpired = false;
                                            }
                                            resolve(rs);
                                        }).catch(function (rs) {
                                        reject(rs);
                                    });
                                });
                            } else {
                                console.log("network hata");
                                reject({
                                    response: response,
                                    headers: headers,
                                    status: {
                                        code: xhr.status,
                                        message: xhr.statusText
                                    }
                                });
                            }
                        }
                    }
                };

                if (useXhr2) {
                    xhr.onerror = function (error) {
                        if (xhr.readyState == 4) {

                            activePage = myApp.getCurrentView().activePage.name;
                            if (activePage === 'conversation') {
                                resolve({
                                    response: {
                                        success: false,
                                        error: error
                                    },
                                    status: null
                                });
                            }
                            else {
                                reject({
                                    response: {
                                        success: false,
                                        error: error
                                    },
                                    status: null
                                });
                                myApp.hideIndicator();
                                myApp.alert("İnternet Bağlantınızı Kontrol Ediniz.", "Bilgi", function () {
                                });
                            }
                        }
                    }
                    ;
                }

            }
        );


        xhr.send(data || null);


        return promise;
    };

    /**
     *
     * @param endpoint
     * @param params
     * @returns {*}
     */
    KivaApi.prototype.get = function (endpoint, params) {
        return this.request("GET", endpoint, params);
    };

    /**
     *
     * @param endpoint
     * @param params
     * @returns {*}
     */
    KivaApi.prototype.post = function (endpoint, params) {
        return this.request("POST", endpoint, params);
    };

    /**
     *
     * @param endpoint
     * @param params
     * @returns {*}
     */
    KivaApi.prototype.put = function (endpoint, params) {
        return this.request("PUT", endpoint, params);
    };

    /**
     *
     * @param endpoint
     * @param params
     * @returns {*}
     */
    KivaApi.prototype.patch = function (endpoint, params) {
        return this.request("PATCH", endpoint, params);
    };

    /**
     *
     * @param endpoint
     * @param params
     * @returns {*}
     */
    KivaApi.prototype.delete = function (endpoint, params) {
        return this.request("DELETE", endpoint, params);
    };

    /**
     *
     * @param endpoint
     * @param params
     * @returns {*}
     */
    KivaApi.prototype.head = function (endpoint, params) {
        return this.request("HEAD", endpoint, params);
    };

    /**
     *
     * @type {KivaApi}
     */
    return new KivaApi("https://api.kivacrm.com/v1");

});
